#ifndef Smith_h
#define Smith_h

/*
SYNTAX      =   [LABEL] WS OPCODE WS OPERAND { WS ',' WS OPERAND } WS.
LABEL       =   IDENTIFIER ':'.
WS          =   { ' ' | '\t' }.
OPCODE      =   DECINT | OPCODEID | EVAL | NORMCODE.
OPCODEID    =   'COPY' | 'STOP' | 'NORM' | 'ADD2' | 'ADD3' | ... .
OPERAND     =   OPCODE | CITE.
CITE        =   'CITE' ( OPCODE | '"' {ASCII} '"' | ''' ASCII ''' ).
NORMCODE    =   '=' | '<>' | '<' | '>' | '>=' | '<='.
EVAL        =   'EVAL' WS '(' WS EXP0 WS ')'
EXP0        =   ['+'|'-'] EXP1 { ('+'|'-') EXP1}.
EXP1        =   EXP2 { ('*'|'/') EXP2}.
EXP2        =   EXP3 | ( '(' EXP0 ')' ).
EXP3        =   DECINT | IDENTIFIER.
IDENTIFIER  =   CHAR { ( CHAR | DECDIGIT ) }.
DECINT      =   DECDIGIT { DECDIGIT }.
DECDIGIT    =   '0' .. '9'.
*/
#define OPCODE_STOP     0
#define OPCODE_CITE     1
#define OPCODE_COPY     2
#define OPCODE_NORM     3
#define OPCODE_ADD      4
#define OPCODE_SUB      5
#define OPCODE_MUL      6
#define OPCODE_DIV      7
#define OPCODE_OUTPUT   8
#define OPCODE_INPUT    9
#define OPCODE_NAND     10

#define NORMCODE_EQUAL          0   // =
#define NORMCODE_NOT_EQUAL      1   // <>
#define NORMCODE_LESS           2   // < 
#define NORMCODE_GREATER        3   // >
#define NORMCODE_LESS_EQUAL     4   // <=
#define NORMCODE_GREATER_EQUAL  5   // >=

#define MAX_CELLS   131072

#ifdef COMPILER_DOES_NOT_SUPPORT_BOOL
typedef int bool;
#define true 1
#define false 0
#endif

typedef long CELL, *PCELLS;

class CellPU 
    {
    public:
        CellPU();
        bool Run();
        bool SingleStep();
        bool Unassemble(long from, long to);
        bool m_bDebugMode;
        long m_nCount;
        long m_nIP;
        CELL m_Cells[MAX_CELLS];
    };

class SFPLabel
    {
    public:
        SFPLabel( SFPLabel** lpRoot, char* lpszName, long lOffset );
        virtual ~SFPLabel();

        SFPLabel* m_lpNext;
        long m_lOffset;
        char* m_lpszName;
    };

class StraightFileParser 
    {
    public:
        virtual bool Read( char* lpszFileName );
        
        bool PARSE_EXPRESSION();
        void LABEL();
        bool CHAR(int* pc);
        bool DECDIGIT(int* pc);
        bool DECINT(long* pl);
        bool NORMCODE(long* pl);
        bool OPCODE();
        bool OPCODEID(long* pl);
        bool OPERAND();
        bool ASCIIOP();
        bool EVAL();
        bool IDENTIFIER(char** ppszName);
        bool EXP0(long* pl);
        bool EXP1(long* pl);
        bool EXP2(long* pl);
        bool EXP3(long* pl);
        bool ResolveReferences();

        long m_lWritePos;
        SFPLabel* m_lpLabels;
        SFPLabel* m_lpReferences;
        SFPLabel* m_lpEvals;
        char* m_lpszExpression;
    };

#endif /* Smith_h */


